/// Demonstration of dynamic page creation using the Smartware Web Server
///
/// Here is a sample for file upload.

#include "SW_HTTPServer.h"
#include "DynamicFileIn.h"
#include "Utility.h"

typedef enum {
    IDLE,
    ACCEPTED,
    REJECTED_TOO_LARGE
} AcceptStatus;

/// DynamicFileIn
///
/// This page lets you submit a file.
///
/// You can see in main how this page was registered.
///
HTTPServer::CallBackResults DynamicFileIn(HTTPServer *svr, HTTPServer::CallBackType type, const char * path, const HTTPServer::namevalue *params, int paramcount)
{
    char buf[100];
    int clSize;     // content-length size to be transferred
    char * p1;      // general purpose pointer for processing the results
    char * p2;      // general purpose pointer for processing the results
    static char * boundary = "";    // for pointing at the boundary marker part of multipart/form-data
    static AcceptStatus accept = IDLE;
    HTTPServer::CallBackResults ret = HTTPServer::ACCEPT_ERROR;
    DIR *d;
    struct dirent *p;

    switch (type) {
    
        case HTTPServer::CONTENT_LENGTH_REQUEST:
            // Here we can find out how much data is headed our way,
            // and choose to accept or reject it.
            clSize = atoi(svr->GetHeaderValue("Content-Length"));
            //printf("Content Len RQST(%d)\r\n", clSize);
            //printf("Content-Type: [%s]\r\n", svr->GetHeaderValue("Content-Type"));
            // also, for the multipart data, we have to learn and track
            // the boundary sequence, so we can parse the big block
            // successfully.
            boundary = strstr((char *)svr->GetHeaderValue("Content-Type"), "boundary=");
            if (boundary)
                boundary += strlen("boundary=");
            //printf("InFile = [%s]\r\n", svr->GetHeaderValue("InFile"));
            // If we're happy with the size, and we have the boundary marker we need
            if (clSize < 3000 && boundary) {    // arbitrarily chosen size to accept
                accept = ACCEPTED;
                ret = HTTPServer::ACCEPT_COMPLETE;
            } else {
                accept = REJECTED_TOO_LARGE;
            }
            break;

        case HTTPServer::DATA_TRANSFER:
            // Here's the big blob of data. We've got some painful work here.
            // The blob looks kind of like this:
            //---
            //Content-Disposition: form-data; name="InFile"; filename="somefile.txt"\r\n
            //Content-Type: text/plain\r\n
            //\r\n
            //Text in the file starts here\r\n
            //and continues as needed.
            //------webKitFormBoundary9A3C872FAC9BE23--\r\n
            //---
            // So, we'll work our way thru by extracting the filename, we'll
            // assume it is text/plain, as in this example.
            // If all looks well, we'll write it to the local file system,
            // and return true.
            printf("Data Transfer\r\n");
            p1 = strchr((char *)path, '\r');
            if (p1) {
                p2 = strstr((char *)path, "filename=\"");
                if (p2) {
                    p2 += strlen("filename=\"");
                    p1 = strchr(p2, '"');
                    if (p1) {
                        char * fn = p2;
                        *p1 = '\0';
                        p1 = strstr(p1+1, "\r\n\r\n");
                        if (p1) {
                            p1 += 4;        // finally, sitting on the start of the file data
                            p2 = strstr(p1, boundary);  // find the end
                            if (p2) {
                                // now, create the file
                                FILE * fp;
                                *p2 = '\0';
                                strcpy(buf, svr->GetWebRoot());
                                strcat(buf, "/");
                                strcat(buf, fn);
                                printf("Writing to [%s]\r\n", buf);
                                fp = fopen(buf, "w");
                                if (fp) {
                                    printf("writing data...\r\n");
                                    fwrite(p1, 1, p2 - p1, fp);
                                    fclose(fp);
                                    printf("done.\r\n");
                                    ret = HTTPServer::ACCEPT_COMPLETE;
                                } else {
                                    printf("failed to open [%s]\r\n", buf);
                                }
                            }
                        }
                    }
                }
            }
            ret = HTTPServer::ACCEPT_COMPLETE;
            break;
            
        case HTTPServer::SEND_PAGE:
            // send the header
            //printf("Sending page\r\n");
            svr->header(200, "OK", svr->GetSupportedType(".htm"));
            // send some data
            svr->send("<html><head><title>Dynamic File Submit</title></head>\r\n");
            svr->send("<body>\r\n");
            svr->send("<h1>Smart WiFly Web Server</h1>\r\n");
            svr->send("This example permits you to upload a small file. Select the file using ");
            svr->send("the normal form gadget and submit it.");
            switch(accept) {
                case IDLE:
                default:
                    break;
                case ACCEPTED:
                    svr->send("<br/><b>Previous submission accepted.</b><br/>\r\n");
                    break;
                case REJECTED_TOO_LARGE:
                    svr->send("<br/><b>Previous submission rejected.</b><br/>\r\n");
                    break;
            }
            // note that to accept a file, the form enctype is multipart/form-data.
            sprintf(buf, "<form method='post' enctype='multipart/form-data' action='%s'>\r\n", path);
            svr->send(buf);
            svr->send("<table border='1'>\r\n");
            svr->send("<tr><td>&nbsp;</td><td>File</td><td><input type='file' name='InFile' size='40'></td></tr>\r\n");
            svr->send("<tr><td>&nbsp;</td><td colspan='2'><input type='submit' value='submit'><input type='reset' value='clear'></td></tr>\r\n");
            svr->send("</table>\r\n");
            svr->send("</form>\r\n");
#if 1
            sprintf(buf, "Directory of [%s]:<br/>\r\n", svr->GetWebRoot());
            svr->send(buf);
            d = opendir(svr->GetWebRoot());
            if ( d != NULL ) {
                while ( (p = readdir(d)) != NULL ) {
                    sprintf(buf, " - %s<br/>\r\n", p->d_name);
                    svr->send(buf);
                }
                closedir(d);
            } else {
                svr->send("Unable to open directory!");
            }
#endif
            svr->send("<br/><a href='/'>Back to main</a></body></html>\r\n");
            ret = HTTPServer::ACCEPT_COMPLETE;
            break;
            
        default:
            printf("unknown command %d\r\n", type);
            ret = HTTPServer::ACCEPT_ERROR;
            break;
    }
    return ret;
}


